package djmil.cordacheckers.gameproposal;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameProposalContract;
import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatedBy;
import net.corda.v5.application.flows.ResponderFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionValidator;

@InitiatedBy(protocol = "game-proposal-action")
public class ActionResponder implements ResponderFlow {
    private final static Logger log = LoggerFactory.getLogger(CreateResponder.class);

    @CordaInject
    public MemberLookup memberLookup;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;
    
    @Suspendable
    @Override
    public void call(FlowSession session) {
        try {
            UtxoTransactionValidator txValidator = ledgerTransaction -> {
                GameProposalState gameProposal = (GameProposalState) ledgerTransaction.getInputContractStates().get(0);

                GameProposalContract.Action action = ledgerTransaction.getCommands(GameProposalContract.Action.class).get(0);

                checkSessionParticipants(session, gameProposal, action);

                log.info("Verified the transaction - " + ledgerTransaction.getId());
            };

            UtxoSignedTransaction finalizedSignedTransaction = utxoLedgerService
                    .receiveFinality(session, txValidator)
                    .getTransaction();

            log.info("Finished responder flow - " + finalizedSignedTransaction.getId());
        }
        catch(Exception e)
        {
            log.warn("Exceptionally finished responder flow", e);
        }
    }

    @Suspendable
    void checkSessionParticipants(
        FlowSession session,
        GameProposalState gameProposal,
        GameProposalContract.Action action
    ) {
        final MemberX500Name myName = memberLookup.myInfo().getName();
        final MemberX500Name otherName = session.getCounterparty();

        if (action.getReceiver(gameProposal).compareTo(myName) != 0)
            throw new CordaRuntimeException("Bad GameProposal aquirer: expected '"+myName+"', actual '" +gameProposal.getAcquier() +"'");
        
        if (action.getSender(gameProposal).compareTo(otherName) != 0)
            throw new CordaRuntimeException("Bad GameProposal issuer: expected '"+otherName+"', actual '"+gameProposal.getIssuer()+"'");
    }
}
